@@ -59,7 +59,6 @@ def parse_args():
         "--repo",
         help="git repository to work on",
         type=str,
-        required="DWQ_REPO" not in os.environ,
         default=os.environ.get("DWQ_REPO"),
     )
 
@@ -68,7 +67,6 @@ def parse_args():
         "--commit",
         help="git commit to work on",
         type=str,
-        required="DWQ_COMMIT" not in os.environ,
         default=os.environ.get("DWQ_COMMIT"),
     )
 
@@ -98,6 +96,12 @@ def parse_args():
         type=int,
         default=sys.maxsize,
     )
+    parser.add_argument(
+        "-n",
+        "--no-error-output",
+        help="don't print output of errored jobs",
+        action="store_true",
+    )
     parser.add_argument(
         "-t",
         "--tries",
@@ -142,15 +146,7 @@ def parse_args():
         default=[],
     )
     parser.add_argument(
-        "-a",
-        "--asset",
-        help="save specific asset",
-        type=str,
-        action="append",
-        default=[],
-    )
-    parser.add_argument(
-        "-A", "--asset-dir", help="save all assets", type=str, action="store"
+        "-A", "--asset-dir", help="save all assets to specified folder", type=str, action="store"
     )
 
     parser.add_argument(
@@ -183,7 +179,14 @@ def get_env(env):
 
 
 def create_body(args, command, options=None, parent_id=None):
-    body = {"repo": args.repo, "commit": args.commit, "command": command}
+    body = {"command": command}
+
+    if args.repo:
+        body["repo"] = args.repo
+
+    if args.commit:
+        body["commit"] = args.commit
+
     if options:
         body["options"] = options
 
@@ -250,39 +253,25 @@ def dict_dictadd(_dict, key):
 
 def handle_assets(job, args):
     assets = job["result"].get("assets")
-    asset_dir = args.asset_dir
+    asset_dir = os.path.abspath(args.asset_dir)
 
     if assets:
-        asset_map = {}
-        for _asset in args.asset:
-            _split = _asset.split(":")
-            if len(_split) == 1:
-                remote = _asset
-                local = _asset
-            elif len(_split) == 2:
-                remote, local = _split
-            else:
-                print('dwqc: error: invalid asset spec "%s"' % _asset, file=sys.stderr)
-                sys.exit(1)
-
-            asset_map[remote] = local
-
         for remote, data in assets.items():
-            local = asset_map.get(remote)
-            if not local:
-                if asset_dir:
-                    local = os.path.realpath(os.path.join(asset_dir, remote))
-                    if not local.startswith(asset_dir):
-                        print(
-                            'dwqc: warning: asset "%s" is not relative'
-                            ' to "%s", ignoring' % (remote, asset_dir),
-                            file=sys.stderr,
-                        )
-                else:
+            if asset_dir:
+                local = os.path.abspath(os.path.join(asset_dir, remote))
+                if not local.startswith(asset_dir):
                     print(
-                        'dwqc: warning: ignoring asset "%s"' % remote, file=sys.stderr
+                        'dwqc: warning: asset "%s" is not relative'
+                        ' to "%s", ignoring' % (remote, asset_dir), local,
+                        file=sys.stderr,
                     )
                     continue
+            else:
+                print(
+                    'dwqc: warning: ignoring asset "%s"' % remote, file=sys.stderr
+                )
+                continue
+
             util.write_files({local: data})
 
 
@@ -290,6 +279,10 @@ def main():
     global verbose
     args = parse_args()
 
+    if (args.repo is None) ^ (args.commit is None):
+        print("dwqc: error: both repo and commit must be specified!", file=sys.stderr)
+        sys.exit(1)
+
     signal.signal(signal.SIGTERM, sigterm_handler)
 
     job_queue = args.queue
@@ -473,14 +466,15 @@ def main():
                         job_id = job["job_id"]
                         jobs.remove(job_id)
                         done += 1
+                        _has_passed = job["result"]["status"] in {0, "0", "pass"}
                         # if args.progress:
                         #    vprint("\033[F\033[K", end="")
                         # vprint("dwqc: job %s done. result=%s" % (job["job_id"], job["result"]["status"]))
                         if not args.quiet:
-                            if args.progress:
-                                print("\033[K", end="")
-                            print(job["result"]["output"], end="")
-                        _has_passed = job["result"]["status"] in {0, "0", "pass"}
+                            if _has_passed or (not args.no_error_output):
+                                if args.progress:
+                                    print("\033[K", end="")
+                                print(job["result"]["output"], end="")
                         if _has_passed:
                             passed += 1
                             handle_assets(job, args)
@@ -537,7 +531,7 @@ def main():
                                 )
 
                         if not _has_passed:
-                            if failed > failed_expected:
+                            if failed >= failed_expected:
                                 if (failed - failed_expected) > args.maxfail:
                                     print(
                                         "dwqc: more than %i jobs failed. Exiting."
@@ -559,6 +553,11 @@ def main():
         Job.cancel_all(jobs)
         if args.report:
             Job.add(args.report, {"status": "canceled"})
+
+        if args.outfile:
+            args.outfile.seek(0)
+            args.outfile.write(json.dumps(result_list))
+
         sys.exit(1)
 
     if args.report:
