@@ -10,7 +10,7 @@ import socket
 import traceback
 import multiprocessing
 import shutil
-from subprocess import run, PIPE, STDOUT, TimeoutExpired
+from subprocess import run, PIPE, STDOUT, TimeoutExpired, CalledProcessError
 
 import redis
 from redis.exceptions import ConnectionError, RedisError
@@ -128,8 +128,6 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
                     )
 
                     try:
-                        repo = job.body["repo"]
-                        commit = job.body["commit"]
                         command = job.body["command"]
                     except KeyError:
                         vprint(2, f"{worker_str}: invalid job json body")
@@ -143,6 +141,28 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
 
                     vprint(2, f'{worker_str}: command="{command}"')
 
+                    repo = None
+                    commit = None
+
+                    try:
+                        repo = job.body["repo"]
+                        commit = job.body["commit"]
+                    except KeyError:
+                        pass
+
+                    if (repo is None) ^ (commit is None):
+                        vprint(
+                            2,
+                            f"{worker_str}: invalid job json body, only one of repo and commit specified",
+                        )
+                        job.done(
+                            {
+                                "status": "error",
+                                "output": f'{worker_str} invalid job body: "{job.body}"',
+                            }
+                        )
+                        continue
+
                     exclusive = None
                     try:
                         options = job.body.get("options") or {}
@@ -162,8 +182,6 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
 
                     _env.update(
                         {
-                            "DWQ_REPO": repo,
-                            "DWQ_COMMIT": commit,
                             "DWQ_QUEUE": job.queue_name,
                             "DWQ_WORKER": args.name,
                             "DWQ_WORKER_BUILDNUM": str(buildnum),
@@ -178,40 +196,55 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
                     workdir_output = None
                     workdir_error = None
                     try:
-                        try:
-                            (workdir, workdir_output) = gitjobdir.get(
-                                repo, commit, exclusive=exclusive or str(n)
-                            )
-                        except subprocess.CalledProcessError as e:
-                            workdir_error = (
-                                f"{worker_str}: error getting jobdir. output:\n"
-                                + e.output.decode("utf-8", "backslashreplace")
+                        if repo is not None:
+                            _env.update(
+                                {
+                                    "DWQ_REPO": repo,
+                                    "DWQ_COMMIT": commit,
+                                }
                             )
 
-                        if not workdir:
-                            if job.nacks < options.get("max_retries", 2):
-                                job.nack()
-                                vprint(
-                                    1,
-                                    f"{worker_str}: error getting job dir, requeueing job",
+                            try:
+                                (workdir, workdir_output) = gitjobdir.get(
+                                    repo, commit, exclusive=exclusive or str(n)
                                 )
-                            else:
-                                job.done(
-                                    {
-                                        "status": "error",
-                                        "output": workdir_error
-                                        or f"{worker_str}: error getting jobdir\n",
-                                        "worker": args.name,
-                                        "runtime": 0,
-                                        "body": job.body,
-                                    }
+                            except CalledProcessError as e:
+                                workdir_error = (
+                                    f"{worker_str}: error getting jobdir. output:\n"
+                                    + e.output.decode("utf-8", "backslashreplace")
                                 )
-                                vprint(
-                                    1,
-                                    f"{worker_str}: cannot get job dir, erroring job",
-                                )
-                            working_set.discard(job.job_id)
-                            continue
+
+                            if not workdir:
+                                if job.nacks < options.get("max_retries", 2):
+                                    job.nack()
+                                    vprint(
+                                        1,
+                                        f"{worker_str}: error getting job dir, requeueing job",
+                                    )
+                                    if workdir_error:
+                                        vprint(
+                                            1,
+                                            f'{worker_str}: jobdir error: "{workdir_error}"',
+                                        )
+                                else:
+                                    job.done(
+                                        {
+                                            "status": "error",
+                                            "output": workdir_error
+                                            or f"{worker_str}: error getting jobdir\n",
+                                            "worker": args.name,
+                                            "runtime": 0,
+                                            "body": job.body,
+                                        }
+                                    )
+                                    vprint(
+                                        1,
+                                        f"{worker_str}: cannot get job dir, erroring job",
+                                    )
+                                working_set.discard(job.job_id)
+                                continue
+                        else:
+                            workdir = "/tmp"
 
                         workdir_done_at = time.time()
                         files = options.get("files")
@@ -229,8 +262,9 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
                         # subtract time used for checkout and job files
                         timeout -= time.time() - before
 
-                        # be sure to timeout a bit earlier
-                        timeout -= 1
+                        # be sure to timeout a bit earlier, so transmit/network delays
+                        # don't make disque time-out itself.
+                        timeout -= 10
 
                         command_start_at = time.time()
 
@@ -310,7 +344,21 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
 
                             # pack assets
                             try:
-                                asset_files = os.listdir(asset_dir)
+                                asset_files = []
+                                for subdir, _, subdir_files in os.walk(asset_dir):
+                                    # subdir is the absolute folder,
+                                    # subdir_files a list of files.
+                                    # here, we compile a list of file paths
+                                    # relative to asset_dir.
+                                    asset_files.extend(
+                                        [
+                                            os.path.relpath(
+                                                os.path.join(subdir, f), asset_dir
+                                            )
+                                            for f in subdir_files
+                                        ]
+                                    )
+
                                 if asset_files:
                                     before_assets = time.time()
                                     _result.update(
@@ -343,11 +391,12 @@ def worker(n, cmd_server_pool, gitjobdir, args, working_set):
                             )
                             working_set.discard(job.job_id)
                     except Exception as e:
-                        if workdir:
+                        if workdir and repo:
                             gitjobdir.release(workdir)
                         raise e
 
-                    gitjobdir.release(workdir)
+                    if repo:
+                        gitjobdir.release(workdir)
 
         except Exception as e:
             print(f"{worker_str}: uncaught exception")
